#!/bin/bash
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin:~/bin
export PATH

#########################
# 广东堡塔安全技术有限公司
# 堡塔云WAF项目组
# 检查脚本
#########################

PWD_d=`pwd`

install_path=/www
setup_path=$install_path/cloud_waf
waf_path=$setup_path/console

if [ ! -d "$setup_path" ]; then
    echo "没有安装堡塔云WAF,请执行下面的命令安装堡塔云WAF!"
    echo "curl -skSO https://download.bt.cn/cloudwaf/scripts/install_cloudwaf.sh && bash install_cloudwaf.sh"
    exit 1
fi

if [ $(whoami) != "root" ];then
    echo -e "检测到非root用户执行命令，请尝试以下解决方案：\n   1.请切换到root用户执行命令 \n   2.尝试执行以下执行命令：\n     sudo bash $0 $@"
    exit 1
fi

NGINX_NAME="cloudwaf_nginx"
MYSQL_NAME="cloudwaf_mysql"
Containers="$NGINX_NAME $MYSQL_NAME"

nginx_command="docker exec $NGINX_NAME nginx -t"
mysql_command="docker exec $MYSQL_NAME mysql -V"

is64bit=$(getconf LONG_BIT)
if [ "${is64bit}" != '64' ];then
    echo -e "\033[31m 抱歉, 堡塔云WAF不支持32位系统, 请使用64位系统! \033[0m"
    exit 1
fi

S390X_CHECK=$(uname -a|grep s390x)
if [ "${S390X_CHECK}" ];then
    echo -e "\033[31m 抱歉, 堡塔云WAF不支持s390x架构，请使用x86_64服务器架构 \033[0m"
    exit 1
fi

is_aarch64=$(uname -a|grep aarch64)
if [ "${is_aarch64}" != "" ];then
    echo -e "\033[31m 抱歉, 堡塔云WAF暂不支持aarch64架构，请使用x86_64服务器架构 \033[0m"
    exit 1
fi

cd $waf_path
WAF_Port=$(cat $waf_path/data/.server-port)
chmod 700 $waf_path/CloudWaf
user=$($waf_path/CloudWaf get_username)

btw_host='127.0.0.1'

adminpath=$($waf_path/CloudWaf get_auth_path)
curl_Result(){
    result=$(curl -so /dev/null -m 3 -k --tlsv1.3 -w %{http_code} https://$btw_host:${WAF_Port}${adminpath})
}

Check_Waf_Type(){

    if [ -s $waf_path/data/.cluster-state ]; then
        #echo "堡塔云WAF集群"
        number=$(cat $waf_path/data/.cluster-state)
        if [[ "$number" == "1" ]]; then
            echo "堡塔云WAF集群：主控"
            Master="yes"

        elif [ "$number" == "2" ]; then
            echo "堡塔云WAF集群：被控"
            Slave="yes"

        fi
    
    elif [ -f $waf_path/CloudWaf ] && [ ! -f $waf_path/data/.cluster-state ]; then
        echo "堡塔云WAF单机版"
        Standalone="yes"
    fi

}

local_Test(){
    echo ""
    echo "正在测试127.0.0.1是否可以访问堡塔云WAF管理界面"
    curl_Result
    echo "状态码: $result"   "堡塔云WAF服务器内部访问地址: https://$btw_host:${WAF_Port}${adminpath}"
    #echo "堡塔云WAF服务器内部访问地址: https://$btw_host:${WAF_Port}${adminpath}"
    if [ "$result" == "200" ] || [ "$result" == "403"  ] ; then

        echo -e "\033[32m堡塔云WAF管理界面服务器内部访问正常...\033[0m"

    else

        echo -e "\033[32m堡塔云WAF管理界面服务器内部访问异常,正在尝试重启堡塔云WAF管理界面...\033[0m"
        /etc/init.d/btw admin_restart
        sleep 5
        curl_Result

        if [ "$result" == "200" ] || [ "$result" == "403"  ] ; then
            echo -e "\033[32m堡塔云WAF管理界面服务器内部访问正常...\033[0m"
        else
            echo -e "\033[33m堡塔云WAF管理界面重启后服务器内部访问仍然异常,即将打印堡塔云WAF管理界面错误日志...\033[0m"
            tail -n 20 $waf_path/logs/error.log
            current_date=$(date +"%Y%m/%d")
            tail -n 10 ${waf_path}/logs/debug/${current_date}.log
        fi
    
    fi
}

Server_Network_IP_Test(){
    if [ ! -f "$setup_path/offline.pl" ]; then
        echo "正在获取服务器出口IP中,请稍等..."
        Get_Ip_Address
        
        if [ "${getIpAddress}" != "SERVER_IP" ];then

            btw_host="${getIpAddress}"
            
            echo "正在测试出口IP: ${getIpAddress} 是否可以访问堡塔云WAF管理界面"
            curl_Result
            echo "状态码: $result"
            echo "堡塔云WAF服务器出口IP访问地址: https://$btw_host:${WAF_Port}${adminpath}"
            if [ "$result" != "200" ]; then
                echo -e "\033[32m堡塔云WAF管理界面服务器出口IP异常,正在尝试重启堡塔云WAF管理界面...\033[0m"
                /etc/init.d/btw admin_restart
                sleep 5
                curl_Result
                if [ "$result" != "200" ]; then
                    echo -e "\033[33m堡塔云WAF管理界面重启后服务器出口IP仍然异常,即将打印堡塔云WAF管理界面错误日志...\033[0m"
                    tail -n 20 $waf_path/logs/error.log
                    current_date=$(date +"%Y%m/%d")
                    tail -n 10 ${waf_path}/logs/debug/${current_date}.log
                fi
            else
                echo -e "\033[32m堡塔云WAF管理界面出口IP访问正常...\033[0m"
            fi
        else
            echo "获取服务器出口IP中失败,无法检查."
        fi            
    else
        echo "离线模式: 不检查出口IP是否可以访问"
    fi
}

PORT(){
    if Command_Exists ss ; then
        command_port="ss"

    elif Command_Exists netstat ; then
        command_port="netstat"
    else
        echo -e "命令不存在，无法检查端口是否使用"
        command_port=""
    fi

    if [[ ! -z $command_port ]]; then

        if [ -d "$setup_path" ]; then
            # 排除了openresty docker-proxy
            check_command=$($command_port -ltnp | grep -E ':(80|443|33060)\s' |grep -v openresty|grep -v docker-proxy)
        else
            check_command=$($command_port -ltnp | grep -E ':(80|443|33060)\s')
        fi

        if [[ ! -z "$check_command" ]]; then
            echo "$check_command"
            echo -e "\033[31m 堡塔云WAF需要使用 80 443 33060 端口,系统内已有服务使用,否则无法正常使用,请修改相关程序端口. \033[0m"
            exit 1
        fi
    fi
}

Check_Port(){
    echo "正在检查端口是否占用"
    if ! Command_Exists docker; then
        PORT
    else
        check=$(docker ps |grep -wE "$NGINX_NAME|$MYSQL_NAME")
        if [[ -z "$check" ]]; then
            PORT
        fi
    fi
}

GetSysInfo(){
	if [ -s "/etc/redhat-release" ];then
		SYS_VERSION=$(cat /etc/redhat-release)
	elif [ -s "/etc/issue" ]; then
		SYS_VERSION=$(cat /etc/issue)
	fi
	SYS_INFO=$(uname -a)
	SYS_BIT=$(getconf LONG_BIT)
	MEM_TOTAL=$(free -m|grep Mem|awk '{print $2}')
	CPU_INFO=$(getconf _NPROCESSORS_ONLN)

	echo -e ${SYS_VERSION}
	echo -e Bit:${SYS_BIT} Mem:${MEM_TOTAL}M Core:${CPU_INFO}
	echo -e ${SYS_INFO}
	echo -e "请截图以上报错信息发帖至论坛www.bt.cn/bbs求助"

}

Red_Error(){
	echo '=================================================';
	printf '\033[1;31;40m%b\033[0m\n' "$@";
	GetSysInfo
	exit 1;
}

Check_Disk_Space(){
    df_Th=$(df -Th)
    #echo "$df_Th"
    root_df_Th=$( echo "$df_Th"|grep -w /|awk '{print $6}')
    root_df_Th2=$( echo "$df_Th"|grep -w /|awk '{print $5}')
    a=$(echo $root_df_Th|awk -F "%" '{print $1}')
    echo -e "当前/目录已使用: ${root_df_Th} 可用：${root_df_Th2}"
    if [ $a -ge 98 ];then
        echo -e "当前/目录磁盘剩余空间小于2% 可用：${root_df_Th2},可能是因此导致堡塔云WAF无法正常使用."
    fi
    www_df_Th=$(echo "$df_Th"|grep -w /www|awk '{print $6}')
    www_df_Th2=$(echo "$df_Th"|grep -w /www|awk '{print $5}')
    b=$(echo $www_df_Th|awk -F "%" '{print $1}')
    if [[ ! -z $www_df_Th ]]; then
        echo -e "当前/www目录已使用: ${www_df_Th} 可用：${www_df_Th2}"
        if [ $b -ge 98 ];then
            echo -e "当前/www目录磁盘剩余空间小于2% 可用：${www_df_Th2},可能是因此导致堡塔云WAF无法正常使用."
        fi
    fi

    #查看 Inodes
    df_I=$(df -i)
    #echo "$df_I"
    root_df_I=$( echo "$df_I"|grep -w /|awk '{print $5}')
    c=$(echo $root_df_I|awk -F "%" '{print $1}')
    echo -e "当前/目录的Inodes已使用: ${root_df_I}"
    if [ $c -ge 98 ];then
        echo -e "当前/目录磁盘Inodes剩余空间小于2%,可能是因此导致堡塔云WAF无法正常使用."
        echo -e "查询Inodes占用的命令: for i in /*; do echo \$i; find \$i | wc -l; done"
        echo -e "请根据查询结果到对应目录进行处理,注意清理该目录前请备份好数据后再操作."
    fi
    www_df_I=$( echo "$df_I"|grep -w /www|awk '{print $5}')
    b=$(echo $www_df_I|awk -F "%" '{print $1}')
    if [[ ! -z $www_df_I ]]; then
        echo -e "当前/www的Inodes目录已使用: ${www_df_I}"
        if [ $b -ge 98 ];then
            echo -e "当前/www目录磁盘Inodes剩余空间小于2%,可能是因此导致堡塔云WAF无法正常使用."
            echo -e "查询Inodes占用的命令: for i in /www/*; do echo \$i; find \$i | wc -l; done"
            echo -e "请根据查询结果到对应目录进行处理,注意清理该目录前请备份好数据后再操作."
        fi
    fi
}

Get_Pack_Manager(){
	if [ -f "/usr/bin/yum" ] && [ -d "/etc/yum.repos.d" ]; then
		PM="yum"
	elif [ -f "/usr/bin/apt-get" ] && [ -f "/usr/bin/dpkg" ]; then
		PM="apt-get"		
	fi
}

Command_Exists() {
    command -v "$@" >/dev/null 2>&1
}

Check_Sys_Write(){
    echo "正在检测系统关键目录是否可写"
    if [ ! -d "/etc/init.d" ];then
        mkdir -p /etc/init.d
	fi

    if [ ! -d "$install_path" ];then
        mkdir -p $install_path
	fi
	
    Get_Pack_Manager
    if [ "$PM" == "yum" ]; then
        read_dir="$install_path /usr/lib/systemd/system/ /etc/init.d/ /usr/bin/"
    else
        read_dir="$install_path /usr/lib/systemd/system/ /etc/init.d/ /usr/bin/"
    fi
    
    echo "Ture" > /tmp/btw_install_test_111.pl || touch /tmp/btw_install_test_111.pl
    for dir in ${read_dir[@]}
    do
        if [[ -d "$dir" ]]; then
            if [[ ! -f "/tmp/btw_install_test_111.pl" ]]; then
                echo "建立测试 /tmp/btw_install_test_111.pl 文件失败"
                state=0
            else
                \cp /tmp/btw_install_test_111.pl $dir/btw_install_test_111.pl
            fi
            state=$(echo $?)
            if [[ "$state" != "0" ]];then
                echo -e "\033[31m错误：检测到系统关键目录不可写! $read_dir \033[0m"
                echo "1、如果安装了[宝塔系统加固]，请先临时关闭"
                echo "2、如果安装了云锁，请临时关闭[系统加固、文件防护]功能"
                echo "3、如果安装了安全狗，请临时关闭[系统防护]功能"
                echo "4、如果使用了其它安全软件，请先卸载 "
                echo -e "5、如果使用了禁止写入命令，请执行命令取消禁止写入：\n   chattr -iaR $read_dir "

                if [ $(whoami) != "root" ];then
                    echo -e "6、检测到非root用户安装，请尝试以下解决方案：\n   1.请切换到root用户安装 \n   2.尝试执行以下安装命令：\n     sudo bash $0 $@"
                fi

                echo ""
                echo -e "\033[31m解决以上问题后，请尝试重新安装！ \033[0m"
                echo -e "如果无法解决请截图以上报错信息发帖至论坛www.bt.cn/bbs求助"
                exit 1
            else
                rm -f $dir/btw_install_test_111.pl
            fi
        fi
    done

    if [[ -f "/tmp/btw_install_test_111.pl" ]]; then
        rm -f /tmp/btw_install_test_111.pl
    fi
}

Check_Sys_Packs(){
    echo ""
    echo "正在检查系统中是否存在必备的依赖包"
    Packs="wget curl tar gzip"
    if [ -f /usr/bin/which ];then
        for pack in ${Packs[@]}
        do
            check_pack=$(which $pack)
            #echo $check_pack
            if [[ "$check_pack" == "" ]]; then
                    echo -e "\033[31mERROR: $pack 命令不存在，尝试以下解决方法：\033[0m"
                    if [ "$PM" == "yum" ]; then
                        echo 1、使用命令重新安装依赖包：yum reinstall -y ${Packs}
                    else
                        echo 1、使用命令重新安装依赖包：apt-get reinstall -y ${Packs}
                    fi
                    echo -e "2、检查系统源是否可用？尝试更换可用的源参考教程：\n   https://www.bt.cn/bbs/thread-58005-1-1.html "
                    # echo ""
                    # echo -e "\033[31m解决以上问题后，请尝试重新安装！ \033[0m"
                    # echo -e "如果无法解决请截图以上报错信息发帖至论坛www.bt.cn/bbs求助"
                    # exit 1
            fi

        done
    fi
}

get_node_url(){
	if [ ! -f /bin/curl ];then
		if [ "${PM}" = "yum" ]; then
			yum install curl -y
		elif [ "${PM}" = "apt-get" ]; then
			apt-get install curl -y
		fi
	fi

	if [ -f "/www/node.pl" ];then
		download_Url=$(cat /www/node.pl)
		echo "Download node: $download_Url";
		echo '---------------------------------------------';
		return
	fi
	
	echo '---------------------------------------------';
	echo "Selected download node...";
    nodes=(https://dg2.bt.cn https://download.bt.cn https://ctcc1-node.bt.cn https://cmcc1-node.bt.cn https://ctcc2-node.bt.cn https://hk1-node.bt.cn https://na1-node.bt.cn https://jp1-node.bt.cn);
	tmp_file1=/dev/shm/net_test1.pl
	tmp_file2=/dev/shm/net_test2.pl
	[ -f "${tmp_file1}" ] && rm -f ${tmp_file1}
	[ -f "${tmp_file2}" ] && rm -f ${tmp_file2}
	touch $tmp_file1
	touch $tmp_file2
	for node in ${nodes[@]};
	do
		NODE_CHECK=$(curl -k --connect-timeout 3 -m 3 2>/dev/null -w "%{http_code} %{time_total}" ${node}/net_test|xargs)
		RES=$(echo ${NODE_CHECK}|awk '{print $1}')
		NODE_STATUS=$(echo ${NODE_CHECK}|awk '{print $2}')
		TIME_TOTAL=$(echo ${NODE_CHECK}|awk '{print $3 * 1000 - 500 }'|cut -d '.' -f 1)
		if [ "${NODE_STATUS}" == "200" ];then
			if [ $TIME_TOTAL -lt 100 ];then
				if [ $RES -ge 1500 ];then
					echo "$RES $node" >> $tmp_file1
				fi
			else
				if [ $RES -ge 1500 ];then
					echo "$TIME_TOTAL $node" >> $tmp_file2
				fi
			fi

			i=$(($i+1))
			if [ $TIME_TOTAL -lt 100 ];then
				if [ $RES -ge 3000 ];then
					break;
				fi
			fi	
		fi
	done

	NODE_URL=$(cat $tmp_file1|sort -r -g -t " " -k 1|head -n 1|awk '{print $2}')
	if [ -z "$NODE_URL" ];then
		NODE_URL=$(cat $tmp_file2|sort -g -t " " -k 1|head -n 1|awk '{print $2}')
		if [ -z "$NODE_URL" ];then
			NODE_URL='https://download.bt.cn';
		fi
	fi
	rm -f $tmp_file1
	rm -f $tmp_file2
	download_Url=$NODE_URL
	echo "Download node: $download_Url";
	echo '---------------------------------------------';
}

auto_node(){
    #function PermissionJudgment() {
        if [ $UID -ne 0 ]; then
            echo -e "\n$ERROR 权限不足，请使用 Root 用户运行本脚本\n"
            exit 1
        fi
    #}

    clean_bt(){
        sed -i "/bt.cn/d" /etc/hosts
    }

    no_btcheck(){
    #    path_1="/www/server/panel/data"
    #    path_2="/www/server/panel/install"
    #    echo -e "\e[1;31m=================================================================================\e[0m"
    #    for FILE in ${path_1}/userInfo.json ${path_1}/plugin_bin.pl ${path_1}/auth_list.json ${path_2}/public.sh /etc/init.d/bt; do
    #      lsattr ${FILE}
    #    done

    # get_bt=$(grep "bt.cn" /www/server/panel/pyenv/lib/python3.7/urllib/request.py)
    #    if [ "${get_bt}" ]; then
    #       echo "request.py result:"
    #         echo "==================================================================="
    #         grep "bt.cn" -C 3 /www/server/panel/pyenv/lib/python3.7/urllib/request.py
    #         echo "==================================================================="
    #    fi
        echo -e "\033[34m节点连接状态:\033[0m"
        curl -s -m 5 -w "域名:www.bt.cn       状态码:%{http_code}      解析IP:%{remote_ip}\n" https://www.bt.cn -o /dev/null
        curl -s -m 5 -w "域名:api.bt.cn       状态码:%{http_code}      解析IP:%{remote_ip}\n" https://api.bt.cn -o /dev/null
        curl -s -m 5 -w "域名:download.bt.cn  状态码:%{http_code}      解析IP:%{remote_ip}\n" http://download.bt.cn -o /dev/null
        curl -s -m 5 -w "域名:dg1.bt.cn       状态码:%{http_code}      解析IP:%{remote_ip}\n" http://dg1.bt.cn -o /dev/null
        curl -s -m 5 -w "域名:dg2.bt.cn       状态码:%{http_code}      解析IP:%{remote_ip}\n" http://dg2.bt.cn -o /dev/null
        curl -s -m 5 -w "外网测试             状态码:%{http_code}      解析IP:%{remote_ip}\n" https://www.baidu.com -o /dev/null
        echo -e "\033[34m\n系统DNS设置:\033[0m"
        sed -n  '/nameserver/p' /etc/resolv.conf
    }


    _fix_node(){
        host_ip=(42.157.129.47 36.133.1.8 125.90.93.52 124.226.64.155 116.10.184.178 123.129.198.197 103.179.243.14 128.1.164.196 45.76.53.20)
        tmp_file1=/dev/shm/net_test1.pl
        [ -f "${tmp_file1}" ] && rm -f ${tmp_file1}
            touch $tmp_file1
        tmp_file2=/dev/shm/net_test2.pl
        [ -f "${tmp_file2}" ] && rm -f ${tmp_file2}
            touch $tmp_file2
        tmp_file3=/dev/shm/net_test3.pl
        [ -f "${tmp_file3}" ] && rm -f ${tmp_file3}
            touch $tmp_file3
        tmp_file4=/dev/shm/net_test4.pl
        [ -f "${tmp_file4}" ] && rm -f ${tmp_file4}
            touch $tmp_file4
        ser_name="api.bt.cn"
        for host in ${host_ip[@]};
            do
            NODE_CHECK=$(curl --resolv ${ser_name}:443:${host} --connect-timeout 3 -m 3 2>/dev/null -w "%{http_code} %{time_total}" https://${ser_name} -o /dev/null|xargs)
            NODE_STATUS=$(echo ${NODE_CHECK}|awk '{print $1}')
            TIME_TOTAL=$(echo ${NODE_CHECK}|awk '{print $2 * 1000 - 500}'|cut -d '.' -f 1)

            NODE_CHECK_1=$(curl --resolv ${ser_name}:80:${host} --connect-timeout 3 -m 3 2>/dev/null -w "%{http_code} %{time_total}" http://${ser_name} -o /dev/null|xargs)
            NODE_STATUS_1=$(echo ${NODE_CHECK_1}|awk '{print $1}')
            TIME_TOTAL_1=$(echo ${NODE_CHECK_1}|awk '{print $2 * 1000 - 500}'|cut -d '.' -f 1)
            if [ "${NODE_STATUS}" ==  "200" ] && [ "${NODE_STATUS_1}" == "200" ];then
                    echo "$host" >> $tmp_file3
                    if [ $TIME_TOTAL -lt 100 ] && [ $TIME_TOTAL_1 -lt 100 ];then
                                    echo "$host" >> $tmp_file1
                    fi
            fi
            done
        clean_bt
        host_ip1=$(cat $tmp_file1)
        for host1 in ${host_ip1[*]};
            do
            NODE_CHECK_2=$(curl --resolv ${ser_name}:443:${host1} --connect-timeout 3 -m 3 2>/dev/null -w "%{http_code} %{time_total} %{remote_ip}" https://${ser_name} -o /dev/null|xargs)
            echo "$(echo ${NODE_CHECK_2}|awk '{print $2 * 1000 - 500,$3}')" >> ${tmp_file2}
            done
        JIEDIAN=$(cat ${tmp_file2} |sort -r -g -t " " -k 1|tail -n 1|awk '{print $2}'|cut -d '' -f 1)

        host_ip2=$(cat $tmp_file3)
        for host2 in ${host_ip2[*]};
            do
            NODE_CHECK_3=$(curl --resolv ${ser_name}:443:${host2} --connect-timeout 3 -m 3 2>/dev/null -w "%{http_code} %{time_total} %{remote_ip}" https://${ser_name} -o /dev/null|xargs)
            echo "$(echo ${NODE_CHECK_3}|awk '{print $2 * 1000 - 500,$3}')" >> ${tmp_file4}
            done
        JIEDIAN1=$(cat ${tmp_file4} |sort -r -g -t " " -k 1|tail -n 1|awk '{print $2}'|cut -d '' -f 1)

        if [ "$JIEDIAN" == "" ]; then
            echo -e "\033[33m节点指定异常\033[0m"
            echo -e "\033[32m开始第二次自动修复中...\033[0m"
            if [ "$JIEDIAN1" == "" ]; then
                echo -e "\033[31m\n节点二次指定节点异常...\033[0m"
            else
                echo -e "\033[32m\n第二次修复节点成功... \033[0m"
                echo "$JIEDIAN1 www.bt.cn api.bt.cn download.bt.cn dg2.bt.cn dg1.bt.cn" >> /etc/hosts
            fi
        else
            echo "$JIEDIAN www.bt.cn api.bt.cn download.bt.cn dg2.bt.cn dg1.bt.cn" >> /etc/hosts
        fi
        rm -f ${tmp_file1} && rm -f ${tmp_file2} && rm -f ${tmp_file3} && rm -f ${tmp_file4}
    }

    echo -e "\033[32m自动修复节点中... \033[0m"
    _fix_node
    echo -e "\033[32m开始测试节点...请勿中断程序... \033[0m"

    bt_check_01=$(curl -s -m 5 -w "%{http_code}\n" https://www.bt.cn -o /dev/null)
    bt_check_02=$(curl -s -m 5 -w "%{http_code}\n" https://api.bt.cn -o /dev/null)
    bt_check_03=$(curl -s -m 5 -w "%{http_code}\n" http://download.bt.cn -o /dev/null)
    bt_check_04=$(curl -s -m 5 -w "%{http_code}\n" http://dg1.bt.cn -o /dev/null)
    bt_check_05=$(curl -s -m 5 -w "%{http_code}\n" http://dg2.bt.cn -o /dev/null)

    check_ok(){
        echo -e "\033[32mhosts指定节点: $(sed -n  '/bt.cn/p' /etc/hosts|awk '{print $1}')\033[0m"
        echo -e "\033[32m域名解析节点:  $(curl -s -m 5 -w "%{remote_ip}\n" https://www.bt.cn -o /dev/null)\033[0m"
        echo -e "\e[1;34m节点连接测试正常，修复已完成！\033[0m"

    }

    if [ "${bt_check_01}" != 302 ] || [ "${bt_check_02}" != 200 ] || [ "${bt_check_03}" != 200 ] || [ "${bt_check_04}" != 200 ] || [ "${bt_check_05}" != 200 ]; then
        echo "================================================================================="
        clean_bt
        if [ "${bt_check_01}" != 302 ] || [ "${bt_check_02}" != 200 ] || [ "${bt_check_03}" != 200 ] || [ "${bt_check_04}" != 200 ] || [ "${bt_check_05}" != 200 ]; then
            no_btcheck
            echo -e "\e[1;31m\n 修复失败,请将上方红线至此段话显示的所有内容,截图完整上传宝塔论坛或发送给宝塔运维 \033[0m"
            echo -e "\033[31m——————————————————————————————————————————————————————————————————————————————————\033[0m"
        else
            check_ok
        fi    
    else
            check_ok
    fi


}

Check_www_api(){
    if [ ! -f "$setup_path/offline.pl" ]; then
        www_api=(https://www.bt.cn https://api.bt.cn);

        for node1 in ${www_api[@]}; do
            www_api_check=$(curl -so /dev/null --connect-timeout 5 -m 60 -w "%{http_code} %{time_total}" ${node1}|xargs)

            www_api_status=$(echo ${www_api_check}|awk '{print $1}')
            

            if [ "${www_api_status}" == "200" ] || [ "${www_api_status}" == "301" ] || [ "${www_api_status}" == "302" ];then
                
                echo "${node1} 访问正常" > /dev/null
            else
                echo "$www_api_check"
                echo "${node1} 访问失败,正在尝试自动修复节点,请稍等..."
                auto_node
            fi

        done
    fi

}

Bored_waiting(){
    w_time="$wait_time"
    msg="$wait_msg"
    progress="."
    for ((i=0; i<${w_time}; i++))
    do
        printf "$msg %-10s %d\r" "$progress" "$((i+1))"
        sleep 1

        if [ "$progress" == ".........." ]; then
            progress="."
        else
            progress+="."
        fi
    done
    printf "$msg %-10s %d\r" ".........." "$w_time"
    #echo
}

Docker_Start() {
    if Command_Exists docker || [ -f /lib/systemd/system/docker.service ]; then
        if Command_Exists systemctl; then
            # systemctl mask getty@tty1.service
            systemctl enable docker
            systemctl reset-failed docker.service
            systemctl start docker.service
            if [ "$?" != "0" ];then
                systemctl reset-failed docker.service
                systemctl restart docker.service
                if [ "$?" != "0" ];then
                    docker_status=$(systemctl status docker.service)
                    echo "$docker_status"
                    if [ -f /etc/docker/daemon.json ]; then
                        echo "/etc/docker/daemon.json 文件内容:"
                        cat /etc/docker/daemon.json
                        echo -e "\033[33m建议检查以上 /etc/docker/daemon.json 文件内容是否正确？\033[0m"
                    fi
                    #等待100秒动画
                    echo -e "\033[31m启动 docker服务失败，可能启动太快导致\033[0m"
                    wait_msg="正等待100秒后自动重启docker服务"
                    wait_time="100"
                    Bored_waiting
                    systemctl reset-failed docker.service
                    systemctl restart docker.service
                    if [ "$?" != "0" ];then
                        docker_status=$(systemctl status docker.service)
                        echo "$docker_status"
                        Red_Error "docker启动失败" "请尝试等待2分钟执行重启docker命令：systemctl restart docker.service (无内容返回为正常)" "最后重新执行堡塔云WAF安装命令"
                    fi
                fi
            fi
        else
            chkconfig --add docker
            chkconfig --level 2345 docker on
            service docker start
            if [ "$?" != "0" ];then
                service docker.service status
                cat /etc/docker/daemon.json
                Red_Error "docker启动失败。"
            fi
        fi
    fi
}

Docker_Status_Check(){
    if Command_Exists docker ; then
        if Command_Exists systemctl; then
            docker_status=$(systemctl is-active docker)
            # 判断Docker运行状态
            if [ "$docker_status" = "active" ]; then
                echo -e "\033[32m Docker正在运行 \033[0m"
            else
                echo -e "\033[31m Docker 未运行,正在启动... \033[0m"
                Docker_Start
            fi
        fi
    else

        Red_Error "docker 命令不存在,请检查是否安装docker."

    fi
}

Docker_Compose_Check(){

    if Command_Exists docker-compose; then
        echo "docker-compose 存在"
    else 
        if Command_Exists docker; then
            Docker_compose="docker compose version"
            if $Docker_compose; then
                echo "docker compose 存在"
            fi
        fi

    fi
}

Set_Firewall(){
    if [ ! -f "$setup_path/offline.pl" ]; then
        sshPort=$(cat /etc/ssh/sshd_config | grep 'Port '|awk '{print $2}')
        if [ "${PM}" = "apt-get" ]; then
            apt-get install -y ufw
            if [ -f "/usr/sbin/ufw" ] || [ -f "/sbin/ufw" ] ; then
                ufw allow 22/tcp > /dev/null 2>&1
                ufw allow 80/tcp > /dev/null 2>&1
                ufw allow 443/tcp > /dev/null 2>&1
                ufw allow ${WAF_Port}/tcp > /dev/null 2>&1
                ufw allow ${sshPort}/tcp > /dev/null 2>&1
                ufw_status=`ufw status`
                echo y|ufw enable
                ufw default deny
                ufw reload
            fi
        else
            if [ -f "/etc/init.d/iptables" ]; then
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport 22 -j ACCEPT
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport 80 -j ACCEPT
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport 443 -j ACCEPT
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport ${WAF_Port} -j ACCEPT
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport ${sshPort} -j ACCEPT
                iptables -A INPUT -p icmp --icmp-type any -j ACCEPT
                iptables -A INPUT -s localhost -d localhost -j ACCEPT
                iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
                iptables -P INPUT DROP
                service iptables save
                sed -i "s#IPTABLES_MODULES=\"\"#IPTABLES_MODULES=\"ip_conntrack_netbios_ns ip_conntrack_ftp ip_nat_ftp\"#" /etc/sysconfig/iptables-config
                iptables_status=$(service iptables status | grep 'not running')
                if [ "${iptables_status}" == '' ]; then
                    service iptables restart
                fi
            else
                AliyunCheck=$(cat /etc/redhat-release|grep "Aliyun Linux")
                [ "${AliyunCheck}" ] && return
                yum install firewalld -y
                [ "${Centos8Check}" ] && yum reinstall python3-six -y
                systemctl enable firewalld
                systemctl start firewalld
                firewall-cmd --set-default-zone=public > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=22/tcp > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=80/tcp > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=443/tcp > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=${WAF_Port}/tcp > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=${sshPort}/tcp > /dev/null 2>&1
                firewall-cmd --reload
            fi
        fi

    else
        sshPort=$(cat /etc/ssh/sshd_config | grep 'Port '|awk '{print $2}')
        if [ "${PM}" = "apt-get" ]; then
            if [ -f "/usr/sbin/ufw" ] || [ -f "/sbin/ufw" ] ; then
                ufw allow 22/tcp > /dev/null 2>&1
                ufw allow 80/tcp > /dev/null 2>&1
                ufw allow 443/tcp > /dev/null 2>&1
                ufw allow ${WAF_Port}/tcp > /dev/null 2>&1
                ufw allow ${sshPort}/tcp > /dev/null 2>&1
                ufw_status=`ufw status`
                echo y|ufw enable
                ufw default deny
                ufw reload
            fi
        else
            if [ -f "/etc/init.d/iptables" ];then
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport 22 -j ACCEPT
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport 80 -j ACCEPT
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport 443 -j ACCEPT
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport ${WAF_Port} -j ACCEPT
                iptables -I INPUT -p tcp -m state --state NEW -m tcp --dport ${sshPort} -j ACCEPT
                iptables -A INPUT -p icmp --icmp-type any -j ACCEPT
                iptables -A INPUT -s localhost -d localhost -j ACCEPT
                iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT
                iptables -P INPUT DROP
                service iptables save
                sed -i "s#IPTABLES_MODULES=\"\"#IPTABLES_MODULES=\"ip_conntrack_netbios_ns ip_conntrack_ftp ip_nat_ftp\"#" /etc/sysconfig/iptables-config
                iptables_status=$(service iptables status | grep 'not running')
                if [ "${iptables_status}" == '' ];then
                    service iptables restart
                fi
            else
                AliyunCheck=$(cat /etc/redhat-release|grep "Aliyun Linux")
                [ "${AliyunCheck}" ] && return
                [ "${Centos8Check}" ]
                systemctl enable firewalld
                systemctl start firewalld
                firewall-cmd --set-default-zone=public > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=22/tcp > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=80/tcp > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=443/tcp > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=${WAF_Port}/tcp > /dev/null 2>&1
                firewall-cmd --permanent --zone=public --add-port=${sshPort}/tcp > /dev/null 2>&1
                firewall-cmd --reload
            fi
        fi

    fi
}

Check_Firewall(){
    echo ""
    echo "堡塔云WAF管理界面端口: $WAF_Port"

    if [ "${PM}" = "apt-get" ]; then
        if Command_Exists ufw ; then
            ufw_status=$(ufw status)
            ufw_active=$( echo "$ufw_status" | head -n 1)
            ufw_ALLOW_Port=$( echo "$ufw_status" |grep -v v6 |grep -w "$WAF_Port" )
            #echo "$ufw_active"
            if [ "$ufw_active" == "Status: active" ]; then

                if [ -z "$ufw_ALLOW_Port" ]; then
                    echo "检查到系统防火墙未开放堡塔云WAF管理界面端口: $WAF_Port"
                    echo "正在设置系统防火墙..."
                    Set_Firewall
                else
                    echo -e "\033[32m系统防火墙已开放堡塔云WAF管理界面端口: $WAF_Port \033[0m"
                fi
                
            else
                echo "ufw 系统防火墙未开启,忽略检查系统防火墙"
            fi
        fi

    else
        if Command_Exists firewall-cmd ; then
            firewalld_active=$( systemctl is-active firewalld)
            if [ "$firewalld_active" == "active" ]; then
                firewall_ALLOW_Port=$(firewall-cmd --list-ports |grep -w "$WAF_Port")
                if [ -z "$firewall_ALLOW_Port" ]; then
                    echo "检查到系统防火墙未开放堡塔云WAF管理界面端口: $WAF_Port"
                    echo "正在设置系统防火墙..."
                    Set_Firewall
                else
                    echo -e "\033[32m系统防火墙已开放堡塔云WAF管理界面端口: $WAF_Port \033[0m"
                fi
            else
                echo "firewalld 系统防火墙未开启,忽略检查系统防火墙"
            fi

        fi

    fi

}

Get_Ip_Address(){
    if [ ! -f "$setup_path/offline.pl" ]; then
        getIpAddress=""
        getIpAddress=$(curl -sSk --connect-timeout 10 -m 60 https://www.bt.cn/Api/getIpAddress)
        
        if [ -z "${getIpAddress}" ] || [ "${getIpAddress}" = "0.0.0.0" ]; then
            getIpAddress=$(curl -sSk --connect-timeout 10 -m 60 https://api.bt.cn/Api/getIpAddress)
        fi

        if [ -z "${getIpAddress}" ] || [ "${getIpAddress}" = "0.0.0.0" ]; then
            getIpAddress=$(curl -sSk --connect-timeout 10 -m 60 https://4.ipw.cn )
        fi

        if [ -z "${getIpAddress}" ] || [ "${getIpAddress}" = "0.0.0.0" ]; then
            isHosts=$(cat /etc/hosts|grep 'www.bt.cn')
            if [ -z "${isHosts}" ];then
                echo "" >> /etc/hosts
                echo "116.213.43.206 www.bt.cn" >> /etc/hosts
                getIpAddress=$(curl -sSk --connect-timeout 10 -m 60 https://www.bt.cn/Api/getIpAddress)
                if [ -z "${getIpAddress}" ];then
                    sed -i "/bt.cn/d" /etc/hosts
                fi
            fi
        fi

        ipv4_regex="^([0-9]{1,3}\.){3}[0-9]{1,3}$"
        ipv6_regex="^([0-9a-fA-F]{0,4}:){1,7}[0-9a-fA-F]{0,4}$"
        ipv6Address=$(echo ${getIpAddress}|tr -d "[]")

        if [[ ${getIpAddress} =~ ${ipv4_regex} ]]; then
            echo "${getIpAddress}" >/dev/null 2>&1
        elif [[ ${ipv6Address} =~ ${ipv6_regex} ]]; then
            echo "${ipv6Address}" >/dev/null 2>&1
        else
            getIpAddress="SERVER_IP"
        fi

        if [ "${getIpAddress}" != "SERVER_IP" ];then
            echo "${getIpAddress}" > ${waf_path}/data/iplist.txt
            #echo "${getIpAddress}"
        fi

    else
        # 离线安装不获取外网IP
        getIpAddress="SERVER_IP"
    fi

    LOCAL_IP=$(ip addr | grep -E -o '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}' | grep -E -v "^127\.|^255\.|^0\." | head -n 1)

}

Container_Check(){
    nginx_container_exists=0
    mysql_container_exists=0
    #查找$Containers 列表的容器是否存在
    for container in ${Containers}; do
        if [[ ! -z "$container" ]]; then
            #精确匹配容器名称
            container_name=$(docker ps -a --filter "name=^${container}$" --format "{{.Names}}")
            #nginx
            if [ "$container_name" == "$NGINX_NAME" ];then
				nginx_container_exists=1
			fi
            
            #mysql
			if [ "$container_name" == "$MYSQL_NAME" ];then
				mysql_container_exists=1
			fi

        fi
	done
    # echo nginx_container_exists $nginx_container_exists
    # echo mysql_container_exists $mysql_container_exists

    if [ "$nginx_container_exists" == "0" ]; then
        echo "curl -skSO https://download.bt.cn/cloudwaf/scripts/install_cloudwaf.sh && bash install_cloudwaf.sh"
        Red_Error "$NGINX_NAME 容器不存在,请尝试执行上面的命令重新安装堡塔云WAF"

    fi


    if [ "$mysql_container_exists" == "0" ]; then

        echo "curl -skSO https://download.bt.cn/cloudwaf/scripts/install_cloudwaf.sh && bash install_cloudwaf.sh"
        Red_Error "$MYSQL_NAME 容器不存在,请尝试执行上面的命令重新安装堡塔云WAF"

    fi

}

Container_one_Check(){
    #检查指定容器是否正常运行 调用请添加如：container=$NGINX_NAME

    if [ "$container" == "$NGINX_NAME" ]; then
        container_check_command=$nginx_command
    elif [ "$container" == "$MYSQL_NAME" ]; then
        container_check_command=$mysql_command
    fi

        # $container_check_command 2>&1 >/dev/null
        output=$($container_check_command 2>&1 >/dev/null)
}

Container_All_Status(){
    #检查所有容器是否运行
    for container in ${Containers}; do
        #检查指定容器是否正常运行
        echo "正在检查 $container 是否运行"
        container=$container
        Container_one_Check
        if [ "$?" == "0" ]; then
            echo -e "\033[32m$container is running\033[0m"
        else
            echo -e "${container} 未启动,正在尝试重启..."
            systemctl reset-failed docker.service
            systemctl restart docker.service
            if [ -s "/etc/init.d/btw" ]; then
                /etc/init.d/btw ${container}_restart
            elif [ -f "${setup_path}/btw.init" ]; then
                bash ${setup_path}/btw.init ${container}_restart
                \cp -arpf ${setup_path}/btw.init /etc/init.d/btw
                chmod +x /etc/init.d/btw
            else
                docker restart $container
                if [ $? -ne "0" ]; then
                    docker logs --tail=10 $container
                    Red_Error "ERROR: Failed start $container container!" "ERROR: 启动 $container 容器失败！"
                fi
            fi            
        fi
    done 
}

init_CloudWaf(){

    if [ ! -f "$waf_path/CloudWaf" ]; then
        Red_Error "$waf_path/CloudWaf 文件不存在"
    fi
    
    #检查用户是否存在
    cd $waf_path
    chmod 700 $waf_path/CloudWaf
    user_status=$($waf_path/CloudWaf default_user_created)
    # echo "$user_status"
    if [ "$user_status" = "0" ]; then
        echo -e "正在检查用户是否存在..."
        #用户不存在时创建新用户
        default_user=$(cat /dev/urandom | head -n 16 | md5sum | head -c 8 )
        default_password=$(cat /dev/urandom | head -n 16 | md5sum | head -c 8 )
    
        create_user=$($waf_path/CloudWaf create_default_user "${default_user}" "${default_password}")
        get_username=$($waf_path/CloudWaf get_username)
        echo default_user: "$get_username"
        if [[ "$default_user" != "$get_username" ]]; then
            echo default_user: "$default_user"
            echo create_user: "$create_user" 
            echo get_username: "$get_username" 
            Red_Error "ERROR: 创建用户失败，请尝试重新安装！"
        fi

        password_check=$(echo "$create_user" |grep "password:"|awk -F ":" '{print $2}'|awk '{print $1}')
        echo default_password: "$password_check"
        if [ -z "$password_check" ]; then
            echo create_user: "$create_user"
            echo default_password_check: "$password_check"
            Red_Error "ERROR: 创建用户失败，密码为空，请尝试重新安装！"
        fi

        echo -e "\033[31m已经创建新用户与密码：\033[0m"
        echo username: "$get_username"
        echo password: "$password_check"

    fi

    check_database=$(ls "$setup_path/mysql/data/" |grep -w btwaf)
    if [ -z "$check_database" ]; then
        create_database="0"
    fi

    if [ "$mysql_run" == "1" ]; then
        # 从 Verify_Mysql_pass 获取密码
        check_database=$(docker exec -i $MYSQL_NAME mysql -u root -p"$MYSQL_PASS" -e " SHOW DATABASES LIKE 'btwaf';" 2>&1 )
        database=$(echo "$check_database" |grep btwaf)
        if [ -z "$database" ]; then
            create_database="0"
        fi
    fi

    if [ "$create_database" == "0" ] && [ "$mysql_run" == "1" ]; then
    # 数据库不存在时，导入数据
        echo -e "数据库不存在，正在导入..."
        # echo -e "正在导入数据中"
        # wait_msg="请耐心等待"
        # wait_time="10"
        # Bored_waiting

        MAX_RETRIES=10
        retries=0
        start_time=$(date +%s)

        while [ $retries -lt $MAX_RETRIES ]; do
        if docker exec -i $MYSQL_NAME mysql -u root -p"$MYSQL_PASS" -e " SHOW DATABASES;"  >/dev/null 2>&1; then
            echo "正在导入数据中,请耐心等待..."

            output=$(docker exec -i $MYSQL_NAME mysql -u root -p"$MYSQL_PASS" -e "CREATE DATABASE btwaf CHARACTER SET utf8mb4 COLLATE utf8mb4_0900_ai_ci;" 2>&1 )
            if [ $? -ne "0" ]; then
                echo "$output"
                Red_Error "ERROR: Failed create database!" "ERROR: 创建数据库失败！"
            fi

            output=$(docker exec -i $MYSQL_NAME mysql -u root -p"$MYSQL_PASS" btwaf < $setup_path/nginx/conf.d/waf/inc/request_total.sql 2>&1 )
            if [ $? -ne "0" ]; then
                echo "$output"
                Red_Error "ERROR: Failed import database!" "ERROR: 导入数据库失败"
            fi

            output=$(docker exec -i $MYSQL_NAME mysql -u root -p"$MYSQL_PASS" btwaf < $setup_path/nginx/conf.d/waf/inc/btwaf_black.sql 2>&1 )
            if [ $? -ne "0" ]; then
                echo "$output"
                Red_Error "ERROR: Failed import database2!" "ERROR: 导入数据库失败2"
            fi

            #重启nginx
            /etc/init.d/btw ${NGINX_NAME}_restart

            break
        else
            echo "MySQL is not running. mysql未启动完成,10秒后再尝试..."
            wait_msg="请耐心等待"
            wait_time="10"
            Bored_waiting
            retries=$((retries+1))
        fi
        done

        if [ $retries -ge $MAX_RETRIES ]; then
            echo "超过最大重试次数,无法连接到MySQL."
            Red_Error "Error: 导入数据库出错."
        fi

    fi

}

Check_ipfilter(){
    echo ""
    check_ipfilter=$(ps aux|grep -v grep|grep -v panel|grep "ipfilter")
    if [ -z "$check_ipfilter" ]; then
        echo "ipfilter 进程不存在,正在启动中..."
        if [ -s "/etc/init.d/btw" ]; then
            /etc/init.d/btw ip_restart
        elif [ -f "${setup_path}/btw.init" ]; then
            \cp -arpf ${setup_path}/btw.init /etc/init.d/btw
            chmod +x /etc/init.d/btw
            bash ${setup_path}/btw.init ip_restart
        fi
        
        if [ $? -ne "0" ]; then
            echo -e "ERROR: ipfilter出错,正在输出进程,日志信息"
            tail -n 10 ${waf_path}/logs/ipfilter_error.log
            ps aux|grep -v grep|grep -v panel|grep "ipfilter"

            echo "正在尝试重启ipfilter..."
            sleep 5
            /etc/init.d/btw ip_restart
            if [ $? -ne "0" ]; then
                echo "ERROR: 启动ipfilter出错！"
            fi
        fi

    else
        echo -e "\033[32mipfilter运行正常.\033[0m"
    fi
}

Check_admin(){

    check_admin=$(ps aux|grep -v grep| grep -w CloudWaf)
    if [ -z "$check_admin" ]; then
        echo "CloudWaf 进程不存在,正在启动中..."
        if [ -s "/etc/init.d/btw" ]; then
            /etc/init.d/btw admin_restart
        elif [ -f "${setup_path}/btw.init" ]; then
            \cp -arpf ${setup_path}/btw.init /etc/init.d/btw
            chmod +x /etc/init.d/btw
            bash ${setup_path}/btw.init admin_restart
        fi

        if [ $? -ne "0" ]; then
            echo -e "ERROR: 启动堡塔云WAF管理界面出错,正在输出进程,日志信息"
            tail -n 10 ${waf_path}/logs/error.log

            current_date=$(date +"%Y%m/%d")
            tail -n 10 ${waf_path}/logs/debug/${current_date}.log
            echo ""
            ps aux|grep -v grep|grep -E "${waf_path}|CloudWaf"
            ss -tulnp|grep ${WAF_Port}

            echo "正在尝试重启堡塔云WAF管理界面..."
            sleep 5
            /etc/init.d/btw restart
            if [ $? -ne "0" ]; then
                Red_Error "ERROR: 启动堡塔云WAF管理界面出错！"
            fi
        fi
    else
        echo -e "\033[32m堡塔云WAF管理界面运行正常.\033[0m"
    fi
}

Check_Service_Add(){

    if [ $Command_Exists systemctl ]; then
        check_is_enabled=$(systemctl is-enabled btw)
        if [ "$check_is_enabled" != "enabled" ]; then
            systemctl daemon-reload
            systemctl enable btw
        fi
    else
        if [ "${PM}" == "yum" ] || [ "${PM}" == "dnf" ]; then
            chkconfig --add btw
            chkconfig --level 2345 btw on            
        elif [ "${PM}" == "apt-get" ]; then
            update-rc.d btw defaults
        fi
    fi

}

Check_bind_domain(){

    domain=$(cat ${waf_path}/config/sysconfig.json | grep accept_domain |awk -F '\"accept_domain\"' '{print $2}'|awk -F ":" '{print $2}'|awk -F '"' '{print $2}')

    if [ ! -z "$domain" ]; then
        echo ""
        btw_host="$domain"
        echo -e "您已为堡塔云WAF绑定域名,请使用域名访问管理界面:\n  https://$btw_host:${WAF_Port}${adminpath} "
        echo "如无法使用域名访问,请尝试执行命令取消绑定域名: btw 14"
    fi

}

Check_bind_ip(){
    
    sysconfig=$(cat ${waf_path}/config/sysconfig.json)

    bind_ip=$(echo "$sysconfig" | grep -oE '[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+' | tr '\n' ' ')

    if [ ! -z "$bind_ip" ]; then
        echo ""
        echo -e "您已为堡塔云WAF授权IP: "$bind_ip",请使用授权IP访问管理界面"
        echo "如无法使用授权IP访问,请尝试执行命令取消绑定授权IP: btw 15"
    fi

}

Check_System_Info(){
    echo ""
    check_memory=$(free -h | awk '/^Mem:/ {print "物理内存：" $2, "已用：" $3, "可用：" $4, "缓存：" $6} /^Swap:/ {print "虚拟内存：" $2, "已用虚拟内存：" $3, "可用虚拟内存：" $4}')
    if [ -z "$check_memory" ]; then
        free -h | awk '/^内存：/ {print "物理内存：" $2, "已用：" $3, "可用：" $4, "缓存：" $6} /^交换：/ {print "虚拟内存：" $2, "已用虚拟内存：" $3, "可用虚拟内存：" $4}'
    else
        echo "$check_memory"
    fi

    echo " $(cat /proc/cpuinfo|grep -E ^processor|wc -l)核心, $(uptime | awk -F 'load average: ' '{print "系统负载平均值: " $2}')"

    # System_Info=$(hostnamectl)
    # echo "$System_Info" |grep "Operating System"|awk -F":" '{print "操作系统:" $2}'
    # echo "$System_Info" |grep "Kernel"|awk -F":" '{print "内核版本:" $2}'
    # echo "$System_Info" |grep "Architecture"|awk -F":" '{print "系统架构:" $2}'

    System_Info=$(hostnamectl); printf "操作系统: %s, 内核版本: %s, 系统架构: %s\n" "$(echo "$System_Info" | grep "Operating System" | awk -F":" '{print $2}')" "$(echo "$System_Info" | grep "Kernel" | awk -F":" '{print $2}')" "$(echo "$System_Info" | grep "Architecture" | awk -F":" '{print $2}')"

}

Check_Connect_mysql(){

    MAX_RETRIES=10
    retries=0

    while [ $retries -lt $MAX_RETRIES ]; do
        # 使用的错误密码
        output=$(docker exec -i $MYSQL_NAME mysql -u root -p"test" -e " SHOW DATABASES;" 2>&1)
        #output=$(docker exec -i cloudwaf_mysql mysql -u root -p"test" -e " SHOW DATABASES;" 2>&1)
        connect_mysql=$(echo "$output" |grep "connect to local MySQL server")
        
        # 如果 connect_mysql 为空，终止循环
        if [ -z "$connect_mysql" ]; then
            mysql_run="1"
            break
        fi

        retries=$((retries + 1))

        echo "MySQL is not running. mysql未启动完成,5秒后再尝试..."
        echo "$connect_mysql"
        wait_msg="请耐心等待"
        wait_time="5"
        Bored_waiting
        
    done

    if [ $retries -ge $MAX_RETRIES ]; then
        echo "ERROR: 超过最大重试次数,无法连接到MySQL."
        echo -e "\033[31m mysql容器检查命令运行出错,跳过检验mysql密码是否正确. \033[0m"
    fi

}

Verify_Mysql_pass(){
    echo ""
    # 检查是否可以连接到mysql
    Check_Connect_mysql

    if [ "$mysql_run" == "1" ]; then
        echo "正在检验mysql密码是否正确"
        if [ -s "$setup_path/nginx/conf.d/waf/config/config.json" ]; then
            #PASS_config=$(cat $setup_path/nginx/conf.d/waf/config/config.json | grep -oP '(?<="password":")[^"]*')
            PASS_config=$(cat $setup_path/nginx/conf.d/waf/config/config.json | sed -nE 's/.*"password":\s*"([^"]*)".*/\1/p')

            if [ ! -z "$PASS_config" ]; then
                output=$(docker exec -i $MYSQL_NAME mysql -u root -p"$PASS_config" -e " SHOW DATABASES;" 2>&1)
                if [[ "$?" == "0" ]]; then
                    Verify_PASS_config="1"
                    MYSQL_PASS="$PASS_config"
                    echo -e "\033[32m PASS_config mysql密码正确\033[0m"
                else
                    echo "$output"
                    echo "获取的PASS_config: $PASS_config"
                    echo -e "\033[31m "$setup_path/nginx/conf.d/waf/config/config.json" 密码错误,无法写入日志 \033[0m"
                fi
            fi
        fi

        if [ "$Verify_PASS_config" != "1" ]; then
            echo "PASS_config 密码错误,正在检查mysql_default.pl"
            if [ -s "$setup_path/nginx/conf.d/waf/mysql_default.pl" ]; then
                PASS_default=$(cat $setup_path/nginx/conf.d/waf/mysql_default.pl)
                if [ ! -z "$PASS_default" ]; then
                    output=$(docker exec -i $MYSQL_NAME mysql -u root -p"$PASS_default" -e " SHOW DATABASES;" 2>&1)
                    if [[ "$?" == "0" ]]; then
                        Verify_PASS_default="1"
                        MYSQL_PASS="$PASS_default"
                        echo "PASS_default内容: $PASS_default"
                        echo -e "\033[32m PASS_default mysql密码正确\033[0m"
                        echo -e "请检查 "$setup_path/nginx/conf.d/waf/config/config.json" 文件中\"password\":后的字密码段是否PASS_default内容一致?"
                        echo -e "如果不一致,请手动修改成一致,再执行命令检查 PASS_config 密码是否正确?"
                        echo -e "如果修改了还是无法连接,请联系我们协助排查: https://www.bt.cn/bbs/ "
                    else
                        echo "$output"
                        echo "获取的PASS_default: $PASS_default"
                        echo -e "\033[31m "$setup_path/nginx/conf.d/waf/mysql_default.pl" 密码错误 \033[0m"
                    fi
                fi
            fi
        fi

        if [ "$Verify_PASS_config" != "1" ] && [ "$Verify_PASS_default" != "1" ]; then
            echo "PASS_default 密码错误,正在检查PASS_inspect"
            if docker ps -a |grep -w $MYSQL_NAME 2>&1 >/dev/null ; then
                MYSQL_ROOT_PASSWORD=$(docker inspect $MYSQL_NAME |grep MYSQL_ROOT_PASSWORD)
                PASS_inspect=$(echo "$MYSQL_ROOT_PASSWORD" | sed -n 's/.*MYSQL_ROOT_PASSWORD=\([^",]*\).*/\1/p')
                if [ ! -z "$PASS_inspect" ]; then
                    output=$(docker exec -i $MYSQL_NAME mysql -u root -p"$PASS_inspect" -e " SHOW DATABASES;" 2>&1)
                    if [[ "$?" == "0" ]]; then
                        Verify_PASS_inspect="1"
                        MYSQL_PASS="$PASS_inspect"
                        echo "PASS_inspect内容: $PASS_inspect"
                        echo "$PASS_inspect" > $setup_path/nginx/conf.d/waf/mysql_default.pl
                        echo -e "\033[32m PASS_inspect mysql密码正确\033[0m"
                        echo -e "请检查 "$setup_path/nginx/conf.d/waf/config/config.json" 文件中\"password\":后的字密码段是否PASS_inspect内容一致?"
                        echo -e "如果不一致,请手动修改成一致,再执行命令检查 PASS_inspect 密码是否正确?"
                        echo -e "如果修改了还是无法连接,请联系我们协助排查: https://www.bt.cn/bbs/ "
                    else
                        echo "$output"
                        echo "获取的PASS_inspect: $PASS_inspect"
                        echo -e "\033[31m PASS_inspect 密码错误 \033[0m"
                        echo "请联系我们协助排查: https://www.bt.cn/bbs/ "
                    fi
                fi
            fi
        fi


        if [ "$Verify_PASS_config" == "1" ]; then
            if [ ! -f "$setup_path/nginx/conf.d/waf/mysql_default.pl" ]; then
                echo "$PASS_config" > "$setup_path/nginx/conf.d/waf/mysql_default.pl"
            else
                PASS_default=$(cat "$setup_path/nginx/conf.d/waf/mysql_default.pl")
                if [ "$PASS_config" != "$PASS_default" ]; then
                    echo "数据库密码不一致,重新写入mysql_default.pl"
                    echo "$PASS_config" > "$setup_path/nginx/conf.d/waf/mysql_default.pl"
                fi
            fi
        fi

        if [ -z "$MYSQL_PASS" ]; then
            echo -e "\033[31m无法获取到 $MYSQL_NAME 密码.\033[0m"
        fi

    fi

}

cloudwaf_check(){

    Get_Pack_Manager
    Docker_Status_Check

    Check_Port

    Container_Check

    # Check_Sys_Write
    Check_System_Info

    Check_Disk_Space

    Check_Sys_Packs

    echo ""
    Container_All_Status

    Verify_Mysql_pass

    init_CloudWaf

    Check_ipfilter
    Check_admin

    Check_Service_Add

    Check_Firewall

    Check_www_api

    local_Test

    Check_bind_domain

    Check_bind_ip

    # Server_Network_IP_Test

    if [ -f "$setup_path/offline.pl" ]; then
        echo "离线模式"
    fi
    version=$(cat $waf_path/.ver)
    echo ""
    echo "堡塔云WAF "$version" 版本"

}

#检查是否存在WAF类型
Check_Waf_Type
if [[ "$Slave" == "yes" ]]; then  
    # 有堡塔云WAF集群：被控
    echo -e "\033[33m不支持在被控上执行堡塔云WAF单机版检查命令。\033[0m"
    exit 1;
fi

if [[ "$Master" == "yes" ]]; then  
    # 有堡塔云WAF集群：主控
    echo -e "\033[33m不支持在主控上执行堡塔云WAF单机版检查命令。\033[0m"
    exit 1;
fi

cloudwaf_check
