
CREATE TABLE `btwaf`.`blocking_ip` (
  `id` int(11) NOT NULL,
  `time` int(11) NOT NULL COMMENT '拦截的时间',
  `server_name` varchar(128) NOT NULL COMMENT '被攻击的域名',
  `host` varchar(128) NOT NULL COMMENT '被攻击的host',
  `ip` varchar(48) NOT NULL COMMENT '攻击者的IP',
  `ip_city` varchar(24) NOT NULL COMMENT 'IP归属地地区',
  `ip_country` varchar(24) NOT NULL COMMENT 'IP归属省份',
  `ip_province` varchar(24) NOT NULL COMMENT 'IP归属国家',
  `ip_longitude` varchar(16) NOT NULL COMMENT 'IP的经度',
  `ip_latitude` varchar(16) NOT NULL COMMENT 'IP的纬度',
  `method` varchar(8) NOT NULL COMMENT '请求类型:GET POST',
  `uri` varchar(128) NOT NULL COMMENT '不带参数的的url',
  `request_uri` varchar(768) NOT NULL COMMENT '带参数的url',
  `user_agent` varchar(512) NOT NULL COMMENT '攻击者的UA',
  `filter_rule` varchar(128) NOT NULL COMMENT '拦截的规则描述	',
  `risk_type` varchar(48) NOT NULL COMMENT '风险类型',
  `incoming_value` text NOT NULL COMMENT '传入的风险值',
  `get_http_log` text NOT NULL COMMENT 'GET拦截的日志内容	',
  `http_log_path` varchar(128) NOT NULL COMMENT 'POST拦截的日志路径	',
  `block_type` varchar(48) NOT NULL COMMENT '封锁的类型',
  `blocking_time` int(11) NOT NULL COMMENT '封锁的时间',
  `block_status` int(11) NOT NULL COMMENT '封锁的状态'
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- --------------------------------------------------------

--
-- 表的结构 `totla_log`
--

CREATE TABLE `btwaf`.`totla_log` (
  `id` int(11) NOT NULL,
  `time` int(11) NOT NULL COMMENT '拦截的时间',
  `server_name` varchar(128) NOT NULL COMMENT '被攻击的域名',
  `ip` varchar(48) NOT NULL COMMENT '攻击者的IP',
  `ip_city` varchar(24) NOT NULL COMMENT 'IP归属地城市',
  `ip_country` varchar(24) NOT NULL COMMENT 'IP归属地国家',
  `ip_province` varchar(24) NOT NULL COMMENT 'IP归属地省份',
  `ip_longitude` varchar(16) NOT NULL COMMENT '经度',
  `ip_latitude` varchar(16) NOT NULL COMMENT '纬度',
  `method` varchar(8) NOT NULL COMMENT '请求类型',
  `request_uri` varchar(768) NOT NULL COMMENT '拦截的request_uri',
  `uri` varchar(128) NOT NULL COMMENT '拦截的uri',
  `user_agent` varchar(512) NOT NULL COMMENT '攻击者IP的Ua',
  `filter_rule` varchar(128) NOT NULL COMMENT '拦截的规则描述',
  `risk_type` varchar(48) NOT NULL COMMENT '风险类型',
  `incoming_value` text NOT NULL COMMENT '传入的风险值',
  `get_http_log` text NOT NULL COMMENT 'GET拦截的日志内容',
  `http_log_path` varchar(128) NOT NULL COMMENT 'POST拦截的日志路径',
  `host` varchar(128) NOT NULL COMMENT '被攻击的host',
  `action` int(11) NOT NULL COMMENT '动作 1=观察 2=拦截'
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;



CREATE TABLE `ip_intercept` (
  `id` int NOT NULL,
  `date` date NOT NULL COMMENT '日期',
  `server_name` varchar(64) NOT NULL DEFAULT '' COMMENT '网站名称',
  `ip` varchar(64) NOT NULL DEFAULT '' COMMENT 'IP地址',
  `country` varchar(64) NOT NULL DEFAULT '' COMMENT '国家',
  `province` varchar(64) NOT NULL DEFAULT '' COMMENT '省份',
  `city` varchar(64) NOT NULL DEFAULT '' COMMENT '城市',
  `request` int DEFAULT '0' COMMENT '拦截次数'
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

--
-- 转储表的索引
--

--
-- 表的索引 `ip_intercept`
--
ALTER TABLE `ip_intercept`
  ADD PRIMARY KEY (`id`),
  ADD KEY `date` (`date`,`server_name`,`ip`),
  ADD KEY `idx_country` (`country`, `date`, `request`, `province`, `city`, `ip`);

--
-- 在导出的表使用AUTO_INCREMENT
--

--
-- 使用表AUTO_INCREMENT `ip_intercept`
--
ALTER TABLE `ip_intercept`
  MODIFY `id` int NOT NULL AUTO_INCREMENT;
COMMIT;


CREATE TABLE `area_intercept` (
  `id` int NOT NULL,
  `date` date NOT NULL COMMENT '日期',
  `country` varchar(64) NOT NULL DEFAULT '' COMMENT '国家',
  `province` varchar(64) NOT NULL DEFAULT '' COMMENT '省份',
  `city` varchar(64) NOT NULL DEFAULT '' COMMENT '城市',
  `server_name` varchar(64) NOT NULL DEFAULT '' COMMENT '网站名称',
  `request` int DEFAULT '0' COMMENT '拦截次数'
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

--
-- 转储表的索引
--

--
-- 表的索引 `area_intercept`
--
ALTER TABLE `area_intercept`
  ADD PRIMARY KEY (`id`),
  ADD KEY `date` (`date`,`country`,`province`,`city`,`server_name`),
  ADD KEY `date_2` (`date`);

--
-- 在导出的表使用AUTO_INCREMENT
--

--
-- 使用表AUTO_INCREMENT `area_intercept`
--
ALTER TABLE `area_intercept`
  MODIFY `id` int NOT NULL AUTO_INCREMENT;
COMMIT;


--
-- 转储表的索引
--

--
-- 表的索引 `blocking_ip`
--
ALTER TABLE `btwaf`.`blocking_ip`
  ADD PRIMARY KEY (`id`),
  ADD KEY `time` (`time`),
  ADD KEY `server_name` (`server_name`),
  ADD KEY `host` (`host`),
  ADD KEY `ip` (`ip`),
  ADD KEY `ip_city` (`ip_city`),
  ADD KEY `ip_country` (`ip_country`),
  ADD KEY `ip_province` (`ip_province`),
  ADD KEY `method` (`method`),
  ADD KEY `uri` (`uri`),
  ADD KEY `request_uri` (`request_uri`),
  ADD KEY `blocking_time` (`blocking_time`),
  ADD KEY `block_type` (`block_type`);

--
-- 表的索引 `totla_log`
--
ALTER TABLE `btwaf`.`totla_log`
  ADD PRIMARY KEY (`id`),
  ADD KEY `time` (`time`),
  ADD KEY `ip` (`ip`),
  ADD KEY `server_name` (`server_name`),
  ADD KEY `ip_city` (`ip_city`),
  ADD KEY `ip_country` (`ip_country`),
  ADD KEY `ip_province` (`ip_province`),
  ADD KEY `method` (`method`),
  ADD KEY `host` (`host`),
  ADD KEY `uri` (`uri`),
  ADD KEY `request_uri` (`request_uri`);

--
-- 在导出的表使用AUTO_INCREMENT
--

--
-- 使用表AUTO_INCREMENT `blocking_ip`
--
ALTER TABLE `btwaf`.`blocking_ip`
  MODIFY `id` int(11) NOT NULL AUTO_INCREMENT;

--
-- 使用表AUTO_INCREMENT `totla_log`
--
ALTER TABLE `btwaf`.`totla_log`
  MODIFY `id` int(11) NOT NULL AUTO_INCREMENT;
COMMIT;
