
-- 创建数据库
CREATE DATABASE IF NOT EXISTS `btwaf` DEFAULT CHARACTER SET utf8mb4 COLLATE utf8mb4_general_ci;


-- IP访问统计表
CREATE TABLE IF NOT EXISTS `btwaf`.`ip_total` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `date` date NOT NULL COMMENT "日期",
    `server_name` varchar(64) DEFAULT "" NOT NULL COMMENT "网站名称",
    `ip` varchar(64) DEFAULT "" NOT NULL COMMENT "IP地址",
    `request` int(11) DEFAULT 0 COMMENT "请求数",
    `is_inland` tinyint(1) DEFAULT 1 COMMENT "是否境内IP",
    PRIMARY KEY (`id`)
    ) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 AUTO_INCREMENT=1;

-- 添加索引
ALTER TABLE `btwaf`.`ip_total` ADD INDEX `date` (`date`);
ALTER TABLE `btwaf`.`ip_total` ADD INDEX `server_name` (`server_name`);
ALTER TABLE `btwaf`.`ip_total` ADD INDEX `ip` (`ip`);
ALTER TABLE `btwaf`.`ip_total` ADD INDEX `is_inland` (`is_inland`);


-- 组合索引
ALTER TABLE `btwaf`.`ip_total` ADD INDEX `date_server_name` (`date`, `server_name`, `is_inland`);



-- 创建请求统计表
CREATE TABLE IF NOT EXISTS `btwaf`.`request_total` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `date` date NOT NULL COMMENT "日期",
    `hour` tinyint(3) DEFAULT 0 COMMENT "小时",
    `minute` tinyint(3) DEFAULT 0 COMMENT "分钟",
    `server_name` varchar(64) DEFAULT "" NOT NULL COMMENT "网站名称",
    `request` int(11) DEFAULT 0 COMMENT "请求数",
    `err_40x` int(11) DEFAULT 0 COMMENT "40x错误",
    `err_500` int(11) DEFAULT 0 COMMENT "500错误",
    `err_502` int(11) DEFAULT 0 COMMENT "502错误",
    `err_503` int(11) DEFAULT 0 COMMENT "503错误",
    `err_504` int(11) DEFAULT 0 COMMENT "504错误",
    `err_499` int(11) DEFAULT 0 COMMENT "499错误",
    `spider_baidu` int(11) DEFAULT 0 COMMENT "百度蜘蛛",
    `spider_google` int(11) DEFAULT 0 COMMENT "谷歌蜘蛛",
    `spider_bing` int(11) DEFAULT 0 COMMENT "必应蜘蛛",
    `spider_sogou` int(11) DEFAULT 0 COMMENT "搜狗蜘蛛",
    `spider_360` int(11) DEFAULT 0 COMMENT "360蜘蛛",
    `spider_other` int(11) DEFAULT 0 COMMENT "其他蜘蛛",
    `send_bytes` int(11) DEFAULT 0 COMMENT "发送字节数",
    `receive_bytes` int(11) DEFAULT 0 COMMENT "接收字节数",
    `sec_request` int(11) DEFAULT 0 COMMENT "每秒请求",
    `sec_send_bytes` int(11) DEFAULT 0 COMMENT "每秒发送字节数",
    `sec_receive_bytes` int(11) DEFAULT 0 COMMENT "每秒接收字节数",
    `avg_proxy_time` int(11) DEFAULT 0 COMMENT "平均回源时间",
    `proxy_count` int(11) DEFAULT 0 COMMENT "回源次数",
    `ip_count` int(11) DEFAULT 0 COMMENT "IP数量",
    `pv_count` int(11) DEFAULT 0 COMMENT "PV数",
    `uv_count` int(11) DEFAULT 0 COMMENT "UV数",
    `pc_count` int(11) DEFAULT 0 COMMENT "PC设备数",
    `mobile_count` int(11) DEFAULT 0 COMMENT "移动设备数",
    PRIMARY KEY (`id`)
    ) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 AUTO_INCREMENT=1;

-- 添加索引
ALTER TABLE `btwaf`.`request_total` ADD INDEX `date` (`date`);
ALTER TABLE `btwaf`.`request_total` ADD INDEX `hour` (`hour`);
ALTER TABLE `btwaf`.`request_total` ADD INDEX `minute` (`minute`);
ALTER TABLE `btwaf`.`request_total` ADD INDEX `server_name` (`server_name`);

-- 组合索引
ALTER TABLE `btwaf`.`request_total` ADD INDEX `date_hour` (`date`, `hour`);
ALTER TABLE `btwaf`.`request_total` ADD INDEX `date_hour_minute` (`date`, `hour`, `minute`);
ALTER TABLE `btwaf`.`request_total` ADD INDEX `date_hour_minute_server_name` (`date`, `server_name`);


-- 创建地区统计表
CREATE TABLE IF NOT EXISTS `btwaf`.`area_total` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `date` date NOT NULL COMMENT "日期",
    `country` varchar(64) DEFAULT "" NOT NULL COMMENT "国家",
    `province` varchar(64) DEFAULT "" NOT NULL COMMENT "省份",
    `city` varchar(64) DEFAULT "" NOT NULL COMMENT "城市",
    `server_name` varchar(64) DEFAULT "" NOT NULL COMMENT "网站名称",
    `request` int(11) DEFAULT 0 COMMENT "请求数",
    PRIMARY KEY (`id`)
    ) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 AUTO_INCREMENT=1;


-- 添加索引
ALTER TABLE `btwaf`.`area_total` ADD INDEX `date` (`date`);
ALTER TABLE `btwaf`.`area_total` ADD INDEX `country` (`country`);
ALTER TABLE `btwaf`.`area_total` ADD INDEX `province` (`province`);
ALTER TABLE `btwaf`.`area_total` ADD INDEX `city` (`city`);
ALTER TABLE `btwaf`.`area_total` ADD INDEX `server_name` (`server_name`);
ALTER TABLE `btwaf`.`area_total` ADD INDEX `request` (`request`);

-- 组合索引
ALTER TABLE `btwaf`.`area_total` ADD INDEX `select_find` (`date`, `server_name`, `country`,`province`, `city`);



-- 创建URI请求统计表, 用于统计每个URI每天的请求数，保留前一天的TOP 100
CREATE TABLE IF NOT EXISTS `btwaf`.`uri_total` (
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `date` date NOT NULL COMMENT "日期",
    `uri` varchar(256) DEFAULT "" NOT NULL COMMENT "URI",
    `method` varchar(12) DEFAULT "GET" NOT NULL COMMENT "请求方法",
    `content_type` VARCHAR(32) DEFAULT "text/html" NOT NULL COMMENT "请求类型",
    `server_name` varchar(64) DEFAULT "" NOT NULL COMMENT "网站名称",
    `request` int(11) DEFAULT 0 COMMENT "请求数",
    PRIMARY KEY (`id`)
    ) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 AUTO_INCREMENT=1;

-- 添加索引
ALTER TABLE `btwaf`.`uri_total` ADD INDEX `date` (`date`);
ALTER TABLE `btwaf`.`uri_total` ADD INDEX `uri` (`uri`);
ALTER TABLE `btwaf`.`uri_total` ADD INDEX `server_name` (`server_name`);
ALTER TABLE `btwaf`.`uri_total` ADD INDEX `request` (`request`);

-- 组合索引
ALTER TABLE `btwaf`.`uri_total` ADD INDEX `date_server_name` (`date`, `server_name`);
ALTER TABLE `btwaf`.`uri_total` ADD INDEX `date_uri_server_name` (`date`, `uri`, `server_name`);

ALTER TABLE `btwaf`.`uri_total` ADD INDEX `server_name_uri_method` (`server_name`,`uri`,`method`);
ALTER TABLE `btwaf`.`uri_total` ADD INDEX `date_uri_server_name_all` (`date`, `uri`, `server_name`,`method`,`content_type`);





-- 创建CC攻击事件表
CREATE TABLE IF NOT EXISTS `btwaf`.`cc_events`(
    `id` int(11) NOT NULL AUTO_INCREMENT,
    `server_name` varchar(64) DEFAULT "" NOT NULL COMMENT "网站名称",
    `reason` varchar(64) DEFAULT "" NOT NULL COMMENT "原因",
    `cc_level` tinyint(2) DEFAULT 0 COMMENT "CC等级",
    `expire_time` int(11) DEFAULT 0 COMMENT "过期时间",
    `event_time` int(11) DEFAULT 0 COMMENT "事件时间",
    PRIMARY KEY (`id`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8mb4 AUTO_INCREMENT=1;

-- 添加索引
ALTER TABLE `btwaf`.`cc_events` ADD INDEX `server_name` (`server_name`);
ALTER TABLE `btwaf`.`cc_events` ADD INDEX `cc_level` (`cc_level`);
ALTER TABLE `btwaf`.`cc_events` ADD INDEX `expire_time` (`expire_time`);
ALTER TABLE `btwaf`.`cc_events` ADD INDEX `event_time` (`event_time`);

-- 组合索引
ALTER TABLE `btwaf`.`cc_events` ADD INDEX `server_name_expire_time` (`server_name`, `expire_time`);
